(define-module (lukutoukkalib binaryformat)
  #:use-module (lukutoukkalib srfi srfi-56)
  #:use-module (srfi srfi-1)
  #:use-module (ice-9 optargs)
  #:export (create-packer create-unpacker))

(define (write-my-char c port endian)
  (write-binary-uint8 (char->integer c) port endian))

(define (read-my-char port endian)
  (integer->char (read-binary-uint8 port endian)))

(define +format-characters+
  `((#\b ,write-binary-sint8 ,read-binary-sint8)
    (#\B ,write-binary-uint8 ,read-binary-uint8)
    (#\s ,write-binary-sint16 ,read-binary-sint16)
    (#\S ,write-binary-uint16 ,read-binary-uint16)
    (#\i ,write-binary-sint32 ,read-binary-sint32)
    (#\I ,write-binary-uint32 ,read-binary-uint32)
    (#\q ,write-binary-sint64 ,read-binary-sint64)
    (#\Q ,write-binary-uint64 ,read-binary-uint64)
    (#\c ,write-my-char ,read-my-char)))


(define* (create-packer fmt #:optional (endian (default-endian)))
  (let ((fmt-list (string->list fmt))
        (writers '()))
    (define (add-writer w) (set! writers (cons w writers)))
    (for-each 
     (lambda (c)
       (let ((selected (assq-ref +format-characters+ c)))
         (if (not selected) (error "Invalid packer key" c))
         (add-writer (lambda (x) ((car selected) x #f endian)))))
     fmt-list)
    (set! writers (reverse! writers))
    (lambda (. args)
      (if (not (= (length args) (length writers)))
          (throw 'wrong-number-of-arguments 
                 (format #f "This packer handles ~a arguments, got ~a." 
                         (length writers) (length args))))
      (let loop ((args args)
                 (writers writers))
        (if (not (null-list? args))
            (begin
              ((car writers) (car args))
              (loop (cdr args) (cdr writers))))))))

(define* (create-unpacker fmt #:optional (endian (default-endian)))
  (let ((fmt-list (string->list fmt))
         (readers '()))
     (define (add-reader r) (set! readers (cons r readers)))
     (for-each 
      (lambda (c)
        (let ((selected (assq-ref +format-characters+ c)))
          (if (not selected) (error "Invalid packer key" c))
          (add-reader (lambda () ((cadr selected) #f endian)))))
      fmt-list)
     (set! readers (reverse! readers))
     (lambda ()
       (let loop ((readers readers)
                  (accumulator '()))
         (if (not (null-list? readers))
             (loop (cdr readers) 
                   (cons ((car readers)) accumulator))
             (reverse! accumulator))))))
