;;;; Copyright 2005 Juri Pakaste
;;;; 
;;;; This file is part of Lukutoukka.
;;;; 
;;;; Lukutoukka is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; Lukutoukka is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with Lukutoukka; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (bookmarks-test)
  #:use-module ((lukutoukkalib bookmarks)
                :select (record-position 
                         get-position 
                         purge-old-positions))
  #:use-module (unit-test)
  #:use-module (oop goops)
  #:export (create-test-suite))

(define-class <bookmarks-test> (<test-case>))

(define-method (test-record-position-to-empty (test-case <bookmarks-test>))
  (let ((result (record-position "foo" 42 '())))
    (assert-equal 1 (length result))
    (let ((bookmark (car result)))
      (assert-equal "foo" (car bookmark))
      (assert-equal 42 (cadr bookmark))
      (assert-true (number? (caddr bookmark))))))

(define-method (test-record-position-to-nonempty (test-case <bookmarks-test>))
  (let ((result (record-position "fnord" 101 '(("asdf" 10 1000000) 
                                               ("zxcv" 232 198786)))))
    (assert-equal 3 (length result))
    (let ((bookmark (car result)))
      (assert-equal "fnord" (car bookmark))
      (assert-equal 101 (cadr bookmark))
      (assert-true (number? (caddr bookmark))))))

(define-method (test-record-with-existing-name (test-case <bookmarks-test>))
  (let ((result (record-position "foo" 37 '(("asdf" 10 323232) ("foo" 123 12323)))))
    (assert-equal 2 (length result))
    (assert-equal "foo" (caar result))
    (assert-equal "asdf" (caadr result))))

(define-method (test-get-position-from-empty (test-case <bookmarks-test>))
  (assert-equal #f (get-position "zap" '())))

(define-method (test-get-position-from-nonempty (test-case <bookmarks-test>))
  (assert-equal 50 (get-position "zap" '(("zap" 50 13429342)))))

(define-method (test-purge-old-positions-from-empty 
                (test-case <bookmarks-test>))
  (assert-equal '() (purge-old-positions 2000 '())))

(define-method (test-purge-old-positions-from-nonempty 
                (test-case <bookmarks-test>))
  (assert-equal '(("flubb" 20 10))
                (purge-old-positions 9
                                     '(("pop" 20 8)
                                       ("flubb" 20 10)))))

(define (create-test-suite) 
  (let ((suite (make <test-suite> #:name "bookmarks-test-suite")))
    (add suite (make <bookmarks-test>))
    suite))
