;;;; Copyright 2005 Juri Pakaste
;;;; 
;;;; This file is part of Lukutoukka.
;;;; 
;;;; Lukutoukka is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; Lukutoukka is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with Lukutoukka; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (lukutoukkalib bookmarks)
  #:use-module (lukutoukkalib utils)
  #:use-module (srfi srfi-1)
  #:use-module (ice-9 receive)
  #:export (find-bookmark store-bookmark))

(define (record-position name position bookmarks)
  (cons (list name position (current-time))
        (filter (lambda (b) (not (equal? (car b) name))) bookmarks)))

(define (get-position name bookmarks)
  (let ((bookmark (find (lambda (bookmark) 
                          (equal? name (car bookmark))) bookmarks)))
    (and bookmark (cadr bookmark))))

(define (purge-old-positions threshold bookmarks)
  (filter (lambda (bookmark) (> (caddr bookmark) threshold)) bookmarks))

(define (find-bookmark name)
  (get-position name (get-bookmarks)))

(define (store-bookmark name position)
  (let ((bookmarks (get-bookmarks)))
    (catch 'file-locked
           (lambda ()
             (with-output-to-user-file 
              (get-bookmark-file)
              (lambda ()
                (write (record-position name position bookmarks))
                #t)))
           (lambda (key . args)
             (format (current-error-port) 
                     "caught file-locked? ~s ~s~%" key args)
             (throw 'bookmark-store-failed)))))

(define (get-bookmarks)
  (with-input-from-user-file (get-bookmark-file) (lambda () (read))))
