;;;; Copyright 2005 Juri Pakaste
;;;; 
;;;; This file is part of Lukutoukka.
;;;; 
;;;; Lukutoukka is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; Lukutoukka is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with Lukutoukka; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (lukutoukkalib parsers plain)
  #:use-module (lukutoukkalib parsers defs)
  #:use-module (lukutoukkalib utils)
  #:use-module (oop goops)
  #:use-module (ice-9 rdelim))

(define-method (parse-port-with-read-char (input <plain-text-source>) consumer)
  (let ((word '())
        (port (source-port input)))
    (define add-to-word (lambda (x) (set! word (cons x word))))
    (define end-word 
             (lambda ()
               (when (> (length word) 0)
                     (let ((word-string (apply string (reverse! word))))
                       (consumer word-string)))
               (set! word '())))
    (let read-loop ((char (read-char port)))
      (if (not (eof-object? char))
          (begin
            (if (char-whitespace? char)
                (end-word)
                (add-to-word char))
            (read-loop (read-char port)))))
    (end-word)))

(define *whitespace* "\f\n\r\t\v ")

(define-method (parse-port (input <plain-text-source>) consumer)
  (let ((port (source-port input)))
    (let read-loop ((word (read-delimited *whitespace* port)))
      (if (not (eof-object? word))
          (begin
            (if (positive? (string-length word))
                (consumer word))
            (read-loop (read-delimited *whitespace* port)))))))
